﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DPS")

L:RegisterTranslations("enUS", function() return {
	["Damage per Second"] = true,
	["DPS"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Damage per Second"] = "每秒伤害",
	["DPS"] = "每秒伤害(DPS)",	
} end)
	
L:RegisterTranslations("zhTW", function() return {
	["Damage per Second"] = "每秒傷害",
	["DPS"] = "每秒傷害(DPS)",
} end)
	
L:RegisterTranslations("koKR", function() return {
	["Damage per Second"] = "초당 피해량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Damage per Second"] = "Dégâts par seconde",
	["DPS"] = "DPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local seconds = nil
local damageDone = nil
local moduleName = "Damage per Second"
local isInCombat = {}
local combatStartTime = {}
local lastShotTime = {}
local isInCombat = {}

local COMBAT_TIMEOUT = 4

local roster = AceLibrary("Roster-2.1")

local parserEvent = {
	eventType = "Damage",
	sourceID_not = false,
}

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 60394 $"):sub(12, -3)) or 1
mod.version = 5
mod.hidePercentage = true
mod.violationCategory = "damage"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterParserEvent(parserEvent, "OnParserEvent")
	mod:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	mod:ScheduleRepeatingEvent( self.CheckPartyCombat, 1, self )
end

function mod:OnReset()
	if type(seconds) == "table" then
		seconds = mod:del(seconds)
	end
	seconds = mod:new()
	
	if type(damageDone) == "table" then
		damageDone = mod:del(damageDone)
	end
	damageDone = mod:new()

	for k in pairs(combatStartTime) do
		combatStartTime[k] = nil
	end
	for k in pairs(lastShotTime) do
		lastShotTime[k] = nil
	end
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Damage per Second"] end
function mod:GetShortDisplayName() return L["DPS"] end

function mod:OnRemoteData(name, input)
	if name and input then
		self.dirty = (input ~= (self.data[name] or 0))
		self.data[name] = input
	end
end

function mod:OnData(name, input)
	if not self:IsLocal(name) and self.data[name] then
		return
	end	
	self.data[name] = nil
	self.dirty = true
end

function mod:IsDirty()
	return self.dirty or next(isInCombat)
end

function mod:IsBroadcastDirty( charName )
	if not charName then
		charName = self.playerName
	end
	return ( self.broadcast[charName] ~= self:GetData(charName) ) or isInCombat[charName]
end

local function getDuration(name)
	local duration = seconds[name] or 0
	if combatStartTime[name] then
		duration = duration + math.max(lastShotTime[name] - combatStartTime[name], 1.5)
	end
	return math.max(duration, 1.5) 
end

local function calculateData(name)
	return math.floor((damageDone[name] or 0) / getDuration(name))
end

local moduleData = {}
function mod:GetData(name)
	if name then
		return self.data[name] or calculateData(name)
	else
		for k, v in pairs( moduleData ) do
			moduleData[k] = nil
		end
		for name in pairs(damageDone) do
			moduleData[name] = calculateData(name)
		end
		for name, value in pairs(self.data) do
			moduleData[name] = value 
		end
		return moduleData
	end
end

function mod:GetBroadcastData( charName )
	if not charName then
		charName = self.playerName
	end
	self.broadcast[charName] = self:GetData(charName)
	return self.broadcast[charName]	
end

function mod:UpdateDPS(name)
	self:OnData(name, true)
end

function mod:ResetDPSTimer(name)
	if combatStartTime[name] then
		seconds[name] = getDuration(name)
		
		combatStartTime[name] = nil
		lastShotTime[name] = nil
		
		self:UpdateDPS(name)
	end	
end

function mod:CheckDPSTimer(name)
	if lastShotTime[name] and lastShotTime[name] + COMBAT_TIMEOUT < GetTime() then
		self:ResetDPSTimer(name)
	end
end

function mod:OnParserEvent( info )
	local name, victim, amount = self:GetSourceVictimAmount(info)

	if Violation:IsAnonymousPetEvent(info) then
		name = self.playerName
	end

	if not name or amount == 0 or not self:AcceptDataFor( name ) then
		return
	end
	
	-- We ignore split damage from known units to known units.
	-- this includes soullink damage from you to your pet.
	-- thanks rophy for pointing this out.
	if victim and (victim == self.playerName or roster:GetUnitIDFromName( victim )) then
		return
	end
	
 	self:CheckDPSTimer(name)
	
 	local newTime = GetTime()
	combatStartTime[name] = combatStartTime[name] or newTime
	lastShotTime[name] = newTime
	damageDone[name] = ( damageDone[name] or 0 ) + amount
	
	self:UpdateDPS(name)
end

-----------------------------------------------------------------------
-- Combat status watching
-----------------------------------------------------------------------

function mod:PLAYER_REGEN_ENABLED()
	mod:ResetDPSTimer( playerName )
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] then
			if not UnitAffectingCombat( unit.unitid ) then
				isInCombat[name] = nil
				mod:ResetDPSTimer( name )
			else
				mod:CheckDPSTimer( name )
			end
		elseif UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
