﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Healing")

L:RegisterTranslations("enUS", function() return {
	["Healing done"] = true,
	["H"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Healing done"] = "治疗输出",
	["H"] = "治疗输出(H)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Healing done"] = "造成治療",
	["H"] = "造成治療(H)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing done"] = "치유량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Healing done"] = "Soins prodigués",
	["H"] = "S",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local roster = AceLibrary("Roster-2.1")
local parserEvent = {
	eventType = "Heal",
	sourceID_not = false,
}

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Healing done"

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName, "AceEvent-2.0")
mod.revision = tonumber(("$Revision: 60394 $"):sub(12, -3)) or 1
mod.version = 5
mod.violationCategory = "healing"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterParserEvent( parserEvent, "OnParserEvent" )
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Healing done"] end
function mod:GetShortDisplayName() return L["H"] end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function mod:OnParserEvent( info )
	local source, victim, amount = self:GetSourceVictimAmount(info)
	
	if not source or not victim or amount == 0 or not self:AcceptDataFor( source ) then
		return
	end

	local victimid = roster:GetUnitIDFromName( victim )
	amount = amount - getOverheal( victimid, amount )
	self:OnData(source, amount)
end
